#include <GL/glew.h>
#include <QImage.h>
#include <QGL.h>
#include <vcg/Point3.h>
#include "CIGrid.h"

using namespace std;
using namespace vcg;

void CIGrid::drawGL()
{
	glPushMatrix();
	float s=2.0f/max(sx,sy);
	glTranslatef(-1.0f,-1.0f,0);
	glScalef(s,s,s);
	if(ti) 
	{
		glEnable(GL_TEXTURE_2D);
	  glBindTexture(GL_TEXTURE_2D,ti);
	}
	vector<CICell>::iterator ci;
	glGetDoublev(GL_MODELVIEW_MATRIX,mm); 
	glGetDoublev(GL_PROJECTION_MATRIX,mp); 
	glGetIntegerv(GL_VIEWPORT,vp);
	for(ci=g.begin();ci!=g.end();++ci)
    (*ci).drawGL();
	glPopMatrix();
}


bool CIGrid::pickGL(Point2i in, Point2i &pos)
{
	float z;
	glReadPixels(in[0],vp[3]-in[1],1,1,GL_DEPTH_COMPONENT ,GL_FLOAT,&z);
	return pickGL(Point3d(in[0],vp[3]-in[1],z),pos);
}

bool CIGrid::pickGL(Point3d in, Point2i &pos)
{
	Point3d o;
	gluUnProject(in[0],in[1],in[2],mm,mp,vp,&o[0],&o[1],&o[2]);
  if(fabs(o[2] ) < .1) 
	{
		qDebug("picked %f %f %f\n",o[0],o[1],o[2]);
		if(o[0]<0 || o[0]>sx ) return false;
		if(o[1]<0 || o[1]>sy ) return false;
		pos[0]=o[0];
		pos[1]=o[1]; 
		
		return true;
	}
	return false;
}

void CIGrid::resize(const int _sx, const int _sy) 
{
	sx=_sx; sy=_sy;
	g.resize(sx*sy);
	int i,j;
	for(i=0;i<sx;++i)
		for(j=0;j<sy;++j)
		{
			G(i,j).V[0]=Point3f(i+0,j+0,0);
			G(i,j).V[1]=Point3f(i+1,j+0,0);
			G(i,j).V[2]=Point3f(i+1,j+1,0);
			G(i,j).V[3]=Point3f(i+0,j+1,0);
			G(i,j).ComputeNormal();
			G(i,j).selected=false;

		}
}

void CICell::drawGL()
{
	if(selected) glColor3f(1,0,0);
	else glColor3f(1,1,1);
	glBegin(GL_QUADS);
	glNormal(n);
	glTexCoord2f(0.05f,0.05f);
	glVertex(V[0]);
	glTexCoord2f(0.95f,0.05f);
	glVertex(V[1]);
	glTexCoord2f(0.95f,0.95f);
	glVertex(V[2]);
	glTexCoord2f(0.05f,0.95f);
	glVertex(V[3]);
	glEnd();
}

void CICell::ComputeNormal()
{
	Point3f e01=V[1]-V[0];
	Point3f e02=V[2]-V[0];
	n=e01^e02;
	n.Normalize();
}

void CIGrid::initGL()
{
		glEnable(GL_TEXTURE_2D);
			glGenTextures(1, &ti);
			QImage tt; 	tt.load(texFileName);
			QImage tx = QGLWidget::convertToGLFormat ( tt);
			glBindTexture(GL_TEXTURE_2D, ti);
			glTexImage2D( GL_TEXTURE_2D, 0, 3, tx.width(), tx.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tx.bits() );
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, tx.width(), tx.height(), GL_RGBA, GL_UNSIGNED_BYTE, tx.bits() );
}

void CIGrid::clearSelection()
{
	vector<CICell>::iterator ii;
	for (ii=g.begin();ii!=g.end();++ii) (*ii).selected=false;
 }

void CIGrid::selectRect(Point2i s, Point2i e)
{
	int i,j;
	for(i=s[0];i<e[0];++i)
		for(j=s[1];j<e[1];++j)
			G(i,j).selected=true;	
}
